(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react')) :
  typeof define === 'function' && define.amd ? define(['exports', 'react'], factory) :
  (global = global || self, factory(global.GeneralStore = {}, global.React));
}(this, (function (exports, React) { 'use strict';

  /**
   * Copyright (c) 2013-present, Facebook, Inc.
   *
   * This source code is licensed under the MIT license found in the
   * LICENSE file in the root directory of this source tree.
   */

  /**
   * Use invariant() to assert state which your program assumes to be true.
   *
   * Provide sprintf-style format (only %s is supported) and arguments
   * to provide information about what broke and what you were
   * expecting.
   *
   * The invariant message will be stripped in production, but the invariant
   * will remain to ensure logic does not differ in production.
   */

  var invariant = function(condition, format, a, b, c, d, e, f) {
    {
      if (format === undefined) {
        throw new Error('invariant requires an error message argument');
      }
    }

    if (!condition) {
      var error;
      if (format === undefined) {
        error = new Error(
          'Minified exception occurred; use the non-minified dev environment ' +
          'for the full error message and additional helpful warnings.'
        );
      } else {
        var args = [a, b, c, d, e, f];
        var argIndex = 0;
        error = new Error(
          format.replace(/%s/g, function() { return args[argIndex++]; })
        );
        error.name = 'Invariant Violation';
      }

      error.framesToPop = 1; // we don't care about invariant's own frame
      throw error;
    }
  };

  var browser = invariant;

  function isDispatcher(dispatcher) {
      return (!!dispatcher &&
          typeof dispatcher === 'object' &&
          typeof dispatcher.register === 'function' &&
          typeof dispatcher.unregister === 'function');
  }
  function enforceDispatcher(dispatcher) {
      browser(isDispatcher(dispatcher), 'expected `dispatcher` to be a `Flux.Dispatcher` or compatible object but' +
          'got `%s` Learn more about the dispatcher interface:' +
          ' https://github.com/HubSpot/general-store#dispatcher-interface', dispatcher);
  }
  function isPayload(payload) {
      return (payload !== null &&
          typeof payload === 'object' &&
          (typeof payload.actionType === 'string' || typeof payload.type === 'string'));
  }

  var instance = null;
  function get() {
      return instance;
  }
  function set(dispatcher) {
      browser(isDispatcher(dispatcher), 'DispatcherInstance.set: Expected dispatcher to be an object' +
          ' with a register method, and an unregister method but got "%s".' +
          ' Learn more about the dispatcher interface:' +
          ' https://github.com/HubSpot/general-store#dispatcher-interface', dispatcher);
      instance = dispatcher;
  }
  function clear() {
      var oldInstance = instance;
      instance = null;
      return oldInstance;
  }

  var DispatcherInstance = /*#__PURE__*/Object.freeze({
    __proto__: null,
    get: get,
    set: set,
    clear: clear
  });

  /*! *****************************************************************************
  Copyright (c) Microsoft Corporation. All rights reserved.
  Licensed under the Apache License, Version 2.0 (the "License"); you may not use
  this file except in compliance with the License. You may obtain a copy of the
  License at http://www.apache.org/licenses/LICENSE-2.0

  THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
  KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
  WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
  MERCHANTABLITY OR NON-INFRINGEMENT.

  See the Apache Version 2.0 License for specific language governing permissions
  and limitations under the License.
  ***************************************************************************** */

  var __assign = function() {
      __assign = Object.assign || function __assign(t) {
          for (var s, i = 1, n = arguments.length; i < n; i++) {
              s = arguments[i];
              for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
          }
          return t;
      };
      return __assign.apply(this, arguments);
  };

  function __spreadArrays() {
      for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
      for (var r = Array(s), k = 0, i = 0; i < il; i++)
          for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
              r[k] = a[j];
      return r;
  }

  var EventHandler = /** @class */ (function () {
      function EventHandler(instance, key) {
          this._key = key;
          this._instance = instance;
      }
      EventHandler.prototype.getKey = function () {
          return this._key;
      };
      EventHandler.prototype.remove = function () {
          if (this._instance === null || this._instance === undefined) {
              return;
          }
          this._instance.removeHandler(this._key);
          this._instance = null;
      };
      return EventHandler;
  }());

  var PREFIX = 'uid_';
  var nextUid = 0;
  function uid() {
      return PREFIX + nextUid++;
  }

  var Event = /** @class */ (function () {
      function Event() {
          this._handlers = {};
      }
      /**
       * Add a subscription to this event
       *
       * @param  callback  run when the event is triggered.
       * @return this
       */
      Event.prototype.addHandler = function (callback) {
          var key = uid();
          this._handlers[key] = callback;
          return new EventHandler(this, key);
      };
      /**
       * Destroys this event. Removes all handlers.
       *
       * @return this
       */
      Event.prototype.remove = function () {
          this._handlers = {};
          return this;
      };
      /**
       * Removes a subscription by key.
       *
       * @param  key   id of the subscription to remove
       * @return this
       */
      Event.prototype.removeHandler = function (key) {
          delete this._handlers[key];
          return this;
      };
      /**
       * @protected
       * Run a handler by key if it exists
       *
       * @param  key  id of the handler to run
       */
      Event.prototype._runHandler = function (key) {
          if (this._handlers.hasOwnProperty(key)) {
              this._handlers[key].call(null);
          }
      };
      /**
       * Run all subscribed handlers.
       *
       * @return this
       */
      Event.prototype.runHandlers = function () {
          Object.keys(this._handlers).forEach(this._runHandler.bind(this));
          return this;
      };
      /**
       * Convenience method for running multiple events.
       *
       * @param  events  a list of events to run.
       */
      Event.runMultiple = function (events) {
          events.forEach(function (evt) { return evt.runHandlers(); });
      };
      return Event;
  }());

  var HINT_LINK = 'Learn more about using the Store API:' +
      ' https://github.com/HubSpot/general-store#using-the-store-api';
  var hasReduxDevTools = typeof window !== 'undefined' && window.__REDUX_DEVTOOLS_EXTENSION__;
  function getNull() {
      return null;
  }
  var Store = /** @class */ (function () {
      function Store(_a) {
          var _this = this;
          var dispatcher = _a.dispatcher, factory = _a.factory, getter = _a.getter, initialState = _a.initialState, name = _a.name, responses = _a.responses;
          this._dispatcher = dispatcher;
          this._factory = factory;
          this._getter = getter;
          this._name = name || 'Store';
          this._state = initialState;
          this._responses = responses;
          this._event = new Event();
          this._uid = uid();
          this._dispatchToken = this._dispatcher.register(this._handleDispatch.bind(this));
          if (hasReduxDevTools) {
              this._devToolsExtension = window.__REDUX_DEVTOOLS_EXTENSION__.connect({
                  name: name || "Store_" + this._uid,
                  instanceId: this._uid,
              });
              this._unsubscribeDevTools = this._devToolsExtension.subscribe(function (message) {
                  if (message.type === 'DISPATCH' &&
                      message.payload.type === 'JUMP_TO_ACTION') {
                      _this._state = JSON.parse(message.state);
                      _this.triggerChange();
                  }
              });
          }
      }
      /**
       * Subscribe to changes on this store.
       *
       * @param  callback  will run every time the store responds to a dispatcher
       * @return this
       */
      Store.prototype.addOnChange = function (callback) {
          browser(typeof callback === 'function', 'Store.addOnChange: expected callback to be a function' +
              ' but got "%s" instead. %s', callback, HINT_LINK);
          return this._event.addHandler(callback);
      };
      /**
       * Returns the store's referenced value
       *
       * @param  ...  accepts any number of params
       * @return any
       */
      Store.prototype.get = function () {
          var args = [];
          for (var _i = 0; _i < arguments.length; _i++) {
              args[_i] = arguments[_i];
          }
          return this._getter.apply(this, __spreadArrays([this._state], args));
      };
      /**
       * @protected
       * Responds to incoming messages from the Dispatcher
       */
      Store.prototype._handleDispatch = function (payload) {
          {
              browser(isPayload(payload), 'Store: expected dispatched payload to be an object with a' +
                  ' property "actionType" containing a string and an optional property' +
                  ' "data" containing any value but got "%s" instead. Learn more about' +
                  ' the dispatcher interface:' +
                  ' https://github.com/HubSpot/general-store#dispatcher-interface');
          }
          var actionType = payload.actionType || payload.type;
          var data = payload.hasOwnProperty('data')
              ? payload.data
              : payload.payload;
          if (!actionType || !this._responses.hasOwnProperty(actionType)) {
              return;
          }
          this._state = this._responses[actionType](this._state, data, actionType, payload);
          this.triggerChange();
          if (this._devToolsExtension) {
              this._devToolsExtension.send(actionType, this._state);
          }
      };
      /**
       * Destroys this instance of the store.
       * Dispatch callback is unregistered. Subscriptions are removed.
       */
      Store.prototype.remove = function () {
          this._dispatcher.unregister(this._dispatchToken);
          this._event.remove();
          this._getter = getNull;
          this._responses = {};
          typeof this._unsubscribeDevTools === 'function' &&
              this._unsubscribeDevTools();
          typeof this._devToolsExtension !== 'undefined' &&
              this._devToolsExtension.unsubscribe();
      };
      Store.prototype.toString = function () {
          return this._name + "<" + this._state + ">";
      };
      /**
       * Runs all of the store's subscription callbacks
       *
       * @return this
       */
      Store.prototype.triggerChange = function () {
          {
              if (!this._dispatcher.isDispatching()) {
                  console.warn('Store: you called store.triggerChange() outside of a' +
                      ' dispatch loop. Send an action trough the dispatcher to' +
                      ' avoid potentailly confusing behavior.');
              }
          }
          this._event.runHandlers();
          return this;
      };
      return Store;
  }());

  function getActionTypes(store) {
      return Object.keys(store._responses) || [];
  }
  function getDispatcher(store) {
      return store._dispatcher;
  }
  function getDispatchToken(store) {
      return store._dispatchToken;
  }
  function getGetter(store) {
      return store._getter;
  }
  function getId(store) {
      return store._uid;
  }
  function getFactory(store) {
      return store._factory;
  }
  function getName(store) {
      return store._name;
  }
  function getResponses(store) {
      return store._responses;
  }
  function getState(store) {
      return store._state;
  }
  function isStore(store) {
      return store instanceof Store;
  }

  var InspectStore = /*#__PURE__*/Object.freeze({
    __proto__: null,
    getActionTypes: getActionTypes,
    getDispatcher: getDispatcher,
    getDispatchToken: getDispatchToken,
    getGetter: getGetter,
    getId: getId,
    getFactory: getFactory,
    getName: getName,
    getResponses: getResponses,
    getState: getState,
    isStore: isStore
  });

  var HINT_LINK$1 = 'Learn more about defining stores:' +
      ' https://github.com/HubSpot/general-store#create-a-store';
  function enforceResponse(existingResponses, actionType, response) {
      browser(typeof actionType === 'string', 'StoreFactory.defineResponses: expected actionType to be a string' +
          ' but got "%s" instead. %s', actionType, HINT_LINK$1);
      browser(!existingResponses.hasOwnProperty(actionType), 'StoreFactory.defineResponses: conflicting resposes for actionType' +
          ' "%s". Only one response can be defined per actionType per Store. %s', actionType, HINT_LINK$1);
      browser(typeof response === 'function', 'StoreFactory.defineResponses: expected response to be a function' +
          ' but got "%s" instead. %s', response);
  }
  function defaultGetInitialState() {
      return undefined;
  }
  function defaultGetter(state) {
      return state;
  }
  var StoreFactory = /** @class */ (function () {
      function StoreFactory(_a) {
          var _b = _a === void 0 ? {} : _a, getter = _b.getter, getInitialState = _b.getInitialState, name = _b.name, responses = _b.responses;
          this._definition = {
              getter: getter || defaultGetter,
              getInitialState: getInitialState || defaultGetInitialState,
              name: name,
              responses: responses || {},
          };
      }
      StoreFactory.prototype.defineGet = function (getter) {
          browser(this._definition.getter === defaultGetter, 'StoreFactory.defineGet: a getter is already defined.');
          return new StoreFactory(__assign(__assign({}, this._definition), { getter: getter }));
      };
      StoreFactory.prototype.defineGetInitialState = function (getInitialState) {
          browser(typeof getInitialState === 'function', 'StoreFactory.defineGetInitialState: getInitialState must be a function.');
          return new StoreFactory(__assign(__assign({}, this._definition), { getInitialState: getInitialState }));
      };
      StoreFactory.prototype.defineName = function (name) {
          var currentName = this._definition.name;
          return new StoreFactory(__assign(__assign({}, this._definition), { name: currentName ? name + "(" + currentName + ")" : name }));
      };
      StoreFactory.prototype.defineResponses = function (newResponses) {
          var responses = this._definition.responses;
          browser(newResponses && typeof newResponses === 'object', 'StoreFactory.defineResponses: newResponses must be an object');
          Object.keys(newResponses).forEach(function (actionType) {
              return enforceResponse(responses, actionType, newResponses[actionType]);
          });
          return new StoreFactory(__assign(__assign({}, this._definition), { responses: __assign(__assign({}, responses), newResponses) }));
      };
      StoreFactory.prototype.defineResponseTo = function (actionTypes, response) {
          return this.defineResponses([].concat(actionTypes).reduce(function (responses, actionType) {
              responses[actionType] = response;
              return responses;
          }, {}));
      };
      StoreFactory.prototype.getDefinition = function () {
          return this._definition;
      };
      StoreFactory.prototype.register = function (dispatcher) {
          dispatcher = dispatcher || get();
          browser(dispatcher !== null && typeof dispatcher === 'object', "StoreFactory.register: you haven't provided a dispatcher instance." +
              ' You can pass an instance to' +
              ' GeneralStore.define().register(dispatcher) or use' +
              ' GeneralStore.DispatcherInstance.set(dispatcher) to set a global' +
              ' instance.' +
              ' https://github.com/HubSpot/general-store#default-dispatcher-instance');
          enforceDispatcher(dispatcher);
          var _a = this._definition, getter = _a.getter, getInitialState = _a.getInitialState, name = _a.name, responses = _a.responses;
          return new Store({
              dispatcher: dispatcher,
              factory: this,
              getter: getter,
              initialState: getInitialState(),
              name: name,
              responses: responses,
          });
      };
      return StoreFactory;
  }());

  var HINT_LINK$2 = 'Learn more about defining stores:' +
      ' https://github.com/HubSpot/general-store#create-a-store';
  function dropFirstArg(func) {
      return function (head) {
          var tail = [];
          for (var _i = 1; _i < arguments.length; _i++) {
              tail[_i - 1] = arguments[_i];
          }
          return func.apply(void 0, tail);
      };
  }
  var StoreSingleton = /** @class */ (function () {
      function StoreSingleton() {
          var _this = this;
          this._facade = null;
          this._factory = new StoreFactory({
              getter: function (state) {
                  var args = [];
                  for (var _i = 1; _i < arguments.length; _i++) {
                      args[_i - 1] = arguments[_i];
                  }
                  if (typeof _this._getter !== 'function') {
                      return undefined;
                  }
                  return _this._getter.apply(_this, args);
              },
          });
          this._getter = null;
      }
      StoreSingleton.prototype.defineGet = function (getter) {
          browser(!this.isRegistered(), 'StoreSingleton.defineGet: this store definition cannot be modified' +
              ' because it has already been registered with a dispatcher. %s', HINT_LINK$2);
          browser(typeof getter === 'function', 'StoreSingleton.defineGet: expected getter to be a function but got' +
              ' "%s" instead. %s', getter, HINT_LINK$2);
          this._getter = getter;
          return this;
      };
      StoreSingleton.prototype.defineName = function (name) {
          this._factory.defineName(name);
          return this;
      };
      StoreSingleton.prototype.defineResponseTo = function (actionTypes, response) {
          browser(!this.isRegistered(), 'StoreSingleton.defineResponseTo: this store definition cannot be' +
              ' modified because is has already been registered with a dispatcher. %s', HINT_LINK$2);
          this._factory = this._factory.defineResponseTo(actionTypes, dropFirstArg(response));
          return this;
      };
      StoreSingleton.prototype.isRegistered = function () {
          return this._facade instanceof Store;
      };
      StoreSingleton.prototype.register = function (dispatcher) {
          browser(typeof this._getter === 'function', 'StoreSingleton.register: a store cannot be registered without a' +
              ' getter. Use GeneralStore.define().defineGet(getter) to define a' +
              ' getter. %s', HINT_LINK$2);
          if (!this._facade) {
              this._facade = this._factory.register(dispatcher);
          }
          return this._facade;
      };
      return StoreSingleton;
  }());

  function oForEach(subject, operation) {
      for (var key in subject) {
          if (subject.hasOwnProperty(key)) {
              operation(subject[key], key);
          }
      }
  }
  function oMap(subject, mapper) {
      var result = {};
      oForEach(subject, function (value, key) {
          result[key] = mapper(value, key);
      });
      return result;
  }
  function oMerge(subject, updates) {
      oForEach(updates, function (value, key) {
          subject[key] = value;
      });
      return subject;
  }
  function oReduce(subject, reducer, initialAcc) {
      var result = initialAcc;
      oForEach(subject, function (value, key) {
          result = reducer(result, value, key);
      });
      return result;
  }
  function isImmutable(obj) {
      return (obj &&
          typeof obj.hashCode === 'function' &&
          typeof obj.equals === 'function');
  }
  function _compareArrays(arr1, arr2) {
      if (arr1.length !== arr2.length) {
          return false;
      }
      if (!arr1.length) {
          return true;
      }
      for (var idx in arr1) {
          if (arr1[idx] !== arr2[idx]) {
              return false;
          }
      }
      return true;
  }
  function shallowEqual(obj1, obj2) {
      if (obj1 === obj2) {
          return true;
      }
      // Special handling for FB Immutables, as they must be handled specially.
      // While this technically means this is deep equality for Immutables,
      // it's better to have a more specific check than an entirely incorrect one.
      if (isImmutable(obj1) && isImmutable(obj2)) {
          return obj1.equals(obj2);
      }
      if (typeof obj1 !== typeof obj2) {
          return false;
      }
      // would have passed === check if both were same falsy value
      if (!obj1 || !obj2) {
          return false;
      }
      if (typeof obj1 !== 'object') {
          return false;
      }
      if (Object.keys(obj1).length !== Object.keys(obj2).length) {
          return false;
      }
      for (var property in obj1) {
          if (obj1.hasOwnProperty(property) && !obj2.hasOwnProperty(property)) {
              return false;
          }
          if (!obj1.hasOwnProperty(property) && obj2.hasOwnProperty(property)) {
              return false;
          }
          if (obj1.hasOwnProperty(property) && obj2.hasOwnProperty(property)) {
              if (isImmutable(obj1[property]) && isImmutable(obj2[property])) {
                  if (!obj1[property].equals(obj2[property])) {
                      return false;
                  }
              }
              else if (Array.isArray(obj1[property])) {
                  return _compareArrays(obj1[property], obj2[property]);
              }
              else if (obj1[property] !== obj2[property]) {
                  return false;
              }
          }
      }
      return true;
  }

  function enforceValidDependencies(dependencies) {
      browser(dependencies && typeof dependencies === 'object', 'expected `dependencies` to be an `object` but got `%s`', dependencies);
      oForEach(dependencies, function (dependency, field) {
          if (dependency instanceof Store) {
              return;
          }
          browser(dependency && typeof dependency === 'object', 'expected `%s` to be an `object` but got `%s`', field, dependency);
          var deref = dependency.deref, stores = dependency.stores;
          browser(typeof deref === 'function', 'expected `%s.deref` to be a function but got `%s`', field, deref);
          browser(Array.isArray(stores), 'expected `%s.stores` to be an Array but got `%s`', field, dependency.stores);
          stores.forEach(function (store, index) {
              browser(store instanceof Store, 'expected `%s.stores.%s` to be a `Store` but got `%s`', field, index, store);
          });
      });
      return dependencies;
  }
  function dependencyPropTypes(dependencies, existingPropTypes) {
      if (existingPropTypes === void 0) { existingPropTypes = {}; }
      var unrelatedPropTypes = oReduce(existingPropTypes, function (keep, type, name) {
          if (!dependencies.hasOwnProperty(name)) {
              keep[name] = type;
          }
          return keep;
      }, {});
      return oReduce(dependencies, function (types, dependency) {
          if (dependency instanceof Store) {
              return types;
          }
          // eslint-disable-next-line react-app/react/forbid-foreign-prop-types
          var propTypes = dependency.propTypes;
          if (!propTypes || typeof propTypes !== 'object') {
              return types;
          }
          return oMerge(types, propTypes);
      }, unrelatedPropTypes);
  }
  function calculate(dependency, props, state) {
      if (dependency instanceof Store) {
          return dependency.get();
      }
      var deref = dependency.deref, stores = dependency.stores;
      return deref(props, state, stores);
  }
  function calculateInitial(dependencies, props, state) {
      return oMap(dependencies, function (dependency) {
          return calculate(dependency, props, state);
      });
  }
  function calculateForDispatch(dependencies, dependencyIndexEntry, props, state) {
      return oMap(dependencyIndexEntry.fields, function (_, field) {
          return calculate(dependencies[field], props, state);
      });
  }
  function makeIndexEntry() {
      return {
          fields: {},
          dispatchTokens: {},
      };
  }
  function makeDependencyIndex(dependencies) {
      enforceValidDependencies(dependencies);
      return oReduce(dependencies, function (index, dep, field) {
          var stores = dep instanceof Store ? [dep] : dep.stores;
          stores.forEach(function (store) {
              getActionTypes(store).forEach(function (actionType) {
                  var entry = index[actionType];
                  if (!entry) {
                      entry = index[actionType] = makeIndexEntry();
                  }
                  var token = getDispatchToken(store);
                  entry.dispatchTokens[token] = true;
                  entry.fields[field] = true;
              });
          });
          return index;
      }, {});
  }

  function makeDisplayName(prefix, BaseComponent) {
      var baseComponentName = BaseComponent.displayName || BaseComponent.name || 'Component';
      return prefix + "(" + baseComponentName + ")";
  }

  function createCommonjsModule(fn, module) {
  	return module = { exports: {} }, fn(module, module.exports), module.exports;
  }

  var reactIs_development = createCommonjsModule(function (module, exports) {



  {
    (function() {

  // The Symbol used to tag the ReactElement-like types. If there is no native Symbol
  // nor polyfill, then a plain number is used for performance.
  var hasSymbol = typeof Symbol === 'function' && Symbol.for;
  var REACT_ELEMENT_TYPE = hasSymbol ? Symbol.for('react.element') : 0xeac7;
  var REACT_PORTAL_TYPE = hasSymbol ? Symbol.for('react.portal') : 0xeaca;
  var REACT_FRAGMENT_TYPE = hasSymbol ? Symbol.for('react.fragment') : 0xeacb;
  var REACT_STRICT_MODE_TYPE = hasSymbol ? Symbol.for('react.strict_mode') : 0xeacc;
  var REACT_PROFILER_TYPE = hasSymbol ? Symbol.for('react.profiler') : 0xead2;
  var REACT_PROVIDER_TYPE = hasSymbol ? Symbol.for('react.provider') : 0xeacd;
  var REACT_CONTEXT_TYPE = hasSymbol ? Symbol.for('react.context') : 0xeace; // TODO: We don't use AsyncMode or ConcurrentMode anymore. They were temporary
  // (unstable) APIs that have been removed. Can we remove the symbols?

  var REACT_ASYNC_MODE_TYPE = hasSymbol ? Symbol.for('react.async_mode') : 0xeacf;
  var REACT_CONCURRENT_MODE_TYPE = hasSymbol ? Symbol.for('react.concurrent_mode') : 0xeacf;
  var REACT_FORWARD_REF_TYPE = hasSymbol ? Symbol.for('react.forward_ref') : 0xead0;
  var REACT_SUSPENSE_TYPE = hasSymbol ? Symbol.for('react.suspense') : 0xead1;
  var REACT_SUSPENSE_LIST_TYPE = hasSymbol ? Symbol.for('react.suspense_list') : 0xead8;
  var REACT_MEMO_TYPE = hasSymbol ? Symbol.for('react.memo') : 0xead3;
  var REACT_LAZY_TYPE = hasSymbol ? Symbol.for('react.lazy') : 0xead4;
  var REACT_BLOCK_TYPE = hasSymbol ? Symbol.for('react.block') : 0xead9;
  var REACT_FUNDAMENTAL_TYPE = hasSymbol ? Symbol.for('react.fundamental') : 0xead5;
  var REACT_RESPONDER_TYPE = hasSymbol ? Symbol.for('react.responder') : 0xead6;
  var REACT_SCOPE_TYPE = hasSymbol ? Symbol.for('react.scope') : 0xead7;

  function isValidElementType(type) {
    return typeof type === 'string' || typeof type === 'function' || // Note: its typeof might be other than 'symbol' or 'number' if it's a polyfill.
    type === REACT_FRAGMENT_TYPE || type === REACT_CONCURRENT_MODE_TYPE || type === REACT_PROFILER_TYPE || type === REACT_STRICT_MODE_TYPE || type === REACT_SUSPENSE_TYPE || type === REACT_SUSPENSE_LIST_TYPE || typeof type === 'object' && type !== null && (type.$$typeof === REACT_LAZY_TYPE || type.$$typeof === REACT_MEMO_TYPE || type.$$typeof === REACT_PROVIDER_TYPE || type.$$typeof === REACT_CONTEXT_TYPE || type.$$typeof === REACT_FORWARD_REF_TYPE || type.$$typeof === REACT_FUNDAMENTAL_TYPE || type.$$typeof === REACT_RESPONDER_TYPE || type.$$typeof === REACT_SCOPE_TYPE || type.$$typeof === REACT_BLOCK_TYPE);
  }

  function typeOf(object) {
    if (typeof object === 'object' && object !== null) {
      var $$typeof = object.$$typeof;

      switch ($$typeof) {
        case REACT_ELEMENT_TYPE:
          var type = object.type;

          switch (type) {
            case REACT_ASYNC_MODE_TYPE:
            case REACT_CONCURRENT_MODE_TYPE:
            case REACT_FRAGMENT_TYPE:
            case REACT_PROFILER_TYPE:
            case REACT_STRICT_MODE_TYPE:
            case REACT_SUSPENSE_TYPE:
              return type;

            default:
              var $$typeofType = type && type.$$typeof;

              switch ($$typeofType) {
                case REACT_CONTEXT_TYPE:
                case REACT_FORWARD_REF_TYPE:
                case REACT_LAZY_TYPE:
                case REACT_MEMO_TYPE:
                case REACT_PROVIDER_TYPE:
                  return $$typeofType;

                default:
                  return $$typeof;
              }

          }

        case REACT_PORTAL_TYPE:
          return $$typeof;
      }
    }

    return undefined;
  } // AsyncMode is deprecated along with isAsyncMode

  var AsyncMode = REACT_ASYNC_MODE_TYPE;
  var ConcurrentMode = REACT_CONCURRENT_MODE_TYPE;
  var ContextConsumer = REACT_CONTEXT_TYPE;
  var ContextProvider = REACT_PROVIDER_TYPE;
  var Element = REACT_ELEMENT_TYPE;
  var ForwardRef = REACT_FORWARD_REF_TYPE;
  var Fragment = REACT_FRAGMENT_TYPE;
  var Lazy = REACT_LAZY_TYPE;
  var Memo = REACT_MEMO_TYPE;
  var Portal = REACT_PORTAL_TYPE;
  var Profiler = REACT_PROFILER_TYPE;
  var StrictMode = REACT_STRICT_MODE_TYPE;
  var Suspense = REACT_SUSPENSE_TYPE;
  var hasWarnedAboutDeprecatedIsAsyncMode = false; // AsyncMode should be deprecated

  function isAsyncMode(object) {
    {
      if (!hasWarnedAboutDeprecatedIsAsyncMode) {
        hasWarnedAboutDeprecatedIsAsyncMode = true; // Using console['warn'] to evade Babel and ESLint

        console['warn']('The ReactIs.isAsyncMode() alias has been deprecated, ' + 'and will be removed in React 17+. Update your code to use ' + 'ReactIs.isConcurrentMode() instead. It has the exact same API.');
      }
    }

    return isConcurrentMode(object) || typeOf(object) === REACT_ASYNC_MODE_TYPE;
  }
  function isConcurrentMode(object) {
    return typeOf(object) === REACT_CONCURRENT_MODE_TYPE;
  }
  function isContextConsumer(object) {
    return typeOf(object) === REACT_CONTEXT_TYPE;
  }
  function isContextProvider(object) {
    return typeOf(object) === REACT_PROVIDER_TYPE;
  }
  function isElement(object) {
    return typeof object === 'object' && object !== null && object.$$typeof === REACT_ELEMENT_TYPE;
  }
  function isForwardRef(object) {
    return typeOf(object) === REACT_FORWARD_REF_TYPE;
  }
  function isFragment(object) {
    return typeOf(object) === REACT_FRAGMENT_TYPE;
  }
  function isLazy(object) {
    return typeOf(object) === REACT_LAZY_TYPE;
  }
  function isMemo(object) {
    return typeOf(object) === REACT_MEMO_TYPE;
  }
  function isPortal(object) {
    return typeOf(object) === REACT_PORTAL_TYPE;
  }
  function isProfiler(object) {
    return typeOf(object) === REACT_PROFILER_TYPE;
  }
  function isStrictMode(object) {
    return typeOf(object) === REACT_STRICT_MODE_TYPE;
  }
  function isSuspense(object) {
    return typeOf(object) === REACT_SUSPENSE_TYPE;
  }

  exports.AsyncMode = AsyncMode;
  exports.ConcurrentMode = ConcurrentMode;
  exports.ContextConsumer = ContextConsumer;
  exports.ContextProvider = ContextProvider;
  exports.Element = Element;
  exports.ForwardRef = ForwardRef;
  exports.Fragment = Fragment;
  exports.Lazy = Lazy;
  exports.Memo = Memo;
  exports.Portal = Portal;
  exports.Profiler = Profiler;
  exports.StrictMode = StrictMode;
  exports.Suspense = Suspense;
  exports.isAsyncMode = isAsyncMode;
  exports.isConcurrentMode = isConcurrentMode;
  exports.isContextConsumer = isContextConsumer;
  exports.isContextProvider = isContextProvider;
  exports.isElement = isElement;
  exports.isForwardRef = isForwardRef;
  exports.isFragment = isFragment;
  exports.isLazy = isLazy;
  exports.isMemo = isMemo;
  exports.isPortal = isPortal;
  exports.isProfiler = isProfiler;
  exports.isStrictMode = isStrictMode;
  exports.isSuspense = isSuspense;
  exports.isValidElementType = isValidElementType;
  exports.typeOf = typeOf;
    })();
  }
  });
  var reactIs_development_1 = reactIs_development.AsyncMode;
  var reactIs_development_2 = reactIs_development.ConcurrentMode;
  var reactIs_development_3 = reactIs_development.ContextConsumer;
  var reactIs_development_4 = reactIs_development.ContextProvider;
  var reactIs_development_5 = reactIs_development.Element;
  var reactIs_development_6 = reactIs_development.ForwardRef;
  var reactIs_development_7 = reactIs_development.Fragment;
  var reactIs_development_8 = reactIs_development.Lazy;
  var reactIs_development_9 = reactIs_development.Memo;
  var reactIs_development_10 = reactIs_development.Portal;
  var reactIs_development_11 = reactIs_development.Profiler;
  var reactIs_development_12 = reactIs_development.StrictMode;
  var reactIs_development_13 = reactIs_development.Suspense;
  var reactIs_development_14 = reactIs_development.isAsyncMode;
  var reactIs_development_15 = reactIs_development.isConcurrentMode;
  var reactIs_development_16 = reactIs_development.isContextConsumer;
  var reactIs_development_17 = reactIs_development.isContextProvider;
  var reactIs_development_18 = reactIs_development.isElement;
  var reactIs_development_19 = reactIs_development.isForwardRef;
  var reactIs_development_20 = reactIs_development.isFragment;
  var reactIs_development_21 = reactIs_development.isLazy;
  var reactIs_development_22 = reactIs_development.isMemo;
  var reactIs_development_23 = reactIs_development.isPortal;
  var reactIs_development_24 = reactIs_development.isProfiler;
  var reactIs_development_25 = reactIs_development.isStrictMode;
  var reactIs_development_26 = reactIs_development.isSuspense;
  var reactIs_development_27 = reactIs_development.isValidElementType;
  var reactIs_development_28 = reactIs_development.typeOf;

  var reactIs = createCommonjsModule(function (module) {

  {
    module.exports = reactIs_development;
  }
  });

  /**
   * Copyright 2015, Yahoo! Inc.
   * Copyrights licensed under the New BSD License. See the accompanying LICENSE file for terms.
   */

  var REACT_STATICS = {
      childContextTypes: true,
      contextType: true,
      contextTypes: true,
      defaultProps: true,
      displayName: true,
      getDefaultProps: true,
      getDerivedStateFromError: true,
      getDerivedStateFromProps: true,
      mixins: true,
      propTypes: true,
      type: true
  };

  var KNOWN_STATICS = {
      name: true,
      length: true,
      prototype: true,
      caller: true,
      callee: true,
      arguments: true,
      arity: true
  };

  var FORWARD_REF_STATICS = {
      '$$typeof': true,
      render: true,
      defaultProps: true,
      displayName: true,
      propTypes: true
  };

  var MEMO_STATICS = {
      '$$typeof': true,
      compare: true,
      defaultProps: true,
      displayName: true,
      propTypes: true,
      type: true
  };

  var TYPE_STATICS = {};
  TYPE_STATICS[reactIs.ForwardRef] = FORWARD_REF_STATICS;

  function getStatics(component) {
      if (reactIs.isMemo(component)) {
          return MEMO_STATICS;
      }
      return TYPE_STATICS[component['$$typeof']] || REACT_STATICS;
  }

  var defineProperty = Object.defineProperty;
  var getOwnPropertyNames = Object.getOwnPropertyNames;
  var getOwnPropertySymbols = Object.getOwnPropertySymbols;
  var getOwnPropertyDescriptor = Object.getOwnPropertyDescriptor;
  var getPrototypeOf = Object.getPrototypeOf;
  var objectPrototype = Object.prototype;

  function hoistNonReactStatics(targetComponent, sourceComponent, blacklist) {
      if (typeof sourceComponent !== 'string') {
          // don't hoist over string (html) components

          if (objectPrototype) {
              var inheritedComponent = getPrototypeOf(sourceComponent);
              if (inheritedComponent && inheritedComponent !== objectPrototype) {
                  hoistNonReactStatics(targetComponent, inheritedComponent, blacklist);
              }
          }

          var keys = getOwnPropertyNames(sourceComponent);

          if (getOwnPropertySymbols) {
              keys = keys.concat(getOwnPropertySymbols(sourceComponent));
          }

          var targetStatics = getStatics(targetComponent);
          var sourceStatics = getStatics(sourceComponent);

          for (var i = 0; i < keys.length; ++i) {
              var key = keys[i];
              if (!KNOWN_STATICS[key] && !(blacklist && blacklist[key]) && !(sourceStatics && sourceStatics[key]) && !(targetStatics && targetStatics[key])) {
                  var descriptor = getOwnPropertyDescriptor(sourceComponent, key);
                  try {
                      // Avoid failures from read-only properties
                      defineProperty(targetComponent, key, descriptor);
                  } catch (e) {}
              }
          }

          return targetComponent;
      }

      return targetComponent;
  }

  var hoistNonReactStatics_cjs = hoistNonReactStatics;

  function waitForStores(dispatcher, tokens) {
      dispatcher.waitFor(tokens);
  }
  function handleDispatch(dispatcher, dependencyIndex, then, payload) {
      var actionType = payload.actionType || payload.type;
      if (!dependencyIndex[actionType]) {
          return;
      }
      var entry = dependencyIndex[actionType];
      waitForStores(dispatcher, Object.keys(entry.dispatchTokens));
      then(entry);
  }

  function useCurrent(value) {
      var ref = React.useRef(value);
      ref.current = value;
      return ref;
  }
  function _useDispatchSubscription(dependencyMap, currentProps, dispatcher, dependencyValue, setDependencyValue) {
      React.useEffect(function () {
          var dependencyIndex = makeDependencyIndex(dependencyMap);
          var dispatchToken = dispatcher.register(handleDispatch.bind(null, dispatcher, dependencyIndex, function (entry) {
              var newValue = calculateForDispatch(dependencyMap, entry, currentProps.current);
              if (!shallowEqual(newValue, dependencyValue)) {
                  setDependencyValue(newValue);
              }
          }));
          return function () {
              dispatcher.unregister(dispatchToken);
          };
      }, [
          currentProps,
          dependencyMap,
          dependencyValue,
          dispatcher,
          setDependencyValue,
      ]);
  }
  function useStoreDependency(dependency, props, dispatcher) {
      if (dispatcher === void 0) { dispatcher = get(); }
      enforceDispatcher(dispatcher);
      var _a = React.useState({
          dependency: calculate(dependency, props),
      }), dependencyValue = _a[0], setDependencyValue = _a[1];
      var currProps = useCurrent(props);
      var dependencyMap = React.useMemo(function () { return ({ dependency: dependency }); }, [dependency]);
      _useDispatchSubscription(dependencyMap, currProps, dispatcher, dependencyValue, setDependencyValue);
      var newValue = { dependency: calculate(dependency, props) };
      if (!shallowEqual(newValue, dependencyValue)) {
          setDependencyValue(newValue);
      }
      return dependencyValue.dependency;
  }

  function useCurrent$1(value) {
      var ref = React.useRef(value);
      ref.current = value;
      return ref;
  }
  function connect(dependencies, dispatcher) {
      if (dispatcher === void 0) { dispatcher = get(); }
      enforceDispatcher(dispatcher);
      return function connector(BaseComponent) {
          var ConnectedComponent = React.forwardRef(function (props, ref) {
              enforceDispatcher(dispatcher);
              var _a = React.useState(function () {
                  return calculateInitial(dependencies, props);
              }), dependencyValue = _a[0], setDependencyValue = _a[1];
              var currProps = useCurrent$1(props);
              _useDispatchSubscription(dependencies, currProps, dispatcher, dependencyValue, setDependencyValue);
              var newValue = oMap(dependencies, function (dep) {
                  return calculate(dep, props, dependencyValue);
              });
              if (!shallowEqual(newValue, dependencyValue)) {
                  setDependencyValue(newValue);
              }
              // TypeScript really doesn't like how I'm typing the output here
              // @ts-ignore
              return React.createElement(BaseComponent, __assign({}, props, dependencyValue, { ref: ref }));
          });
          ConnectedComponent.defaultProps = BaseComponent.defaultProps;
          ConnectedComponent.dependencies = dependencies;
          ConnectedComponent.displayName = makeDisplayName('Connected', BaseComponent);
          ConnectedComponent.propTypes = dependencyPropTypes(dependencies, BaseComponent.propTypes);
          ConnectedComponent.WrappedComponent = BaseComponent;
          return hoistNonReactStatics_cjs(ConnectedComponent, BaseComponent);
      };
  }

  function subscribe(dependencies, dependencyIndex, dispatcher, callback, props, state) {
      if (props === void 0) { props = {}; }
      if (state === void 0) { state = null; }
      var dispatchToken;
      var storeState = {};
      function remove() {
          if (dispatchToken) {
              dispatcher.unregister(dispatchToken);
              dispatchToken = null;
          }
      }
      function handleUpdate(entry) {
          var prevStoreState = storeState;
          try {
              storeState = __assign(__assign({}, prevStoreState), calculateForDispatch(dependencies, entry, props, state));
              callback(null, storeState, prevStoreState, remove);
          }
          catch (error) {
              remove();
              callback(error, storeState, prevStoreState, remove);
          }
      }
      dispatchToken = dispatcher.register(handleDispatch.bind(null, dispatcher, dependencyIndex, handleUpdate));
      try {
          storeState = calculateInitial(dependencies, props, state);
          callback(null, storeState, {}, remove);
      }
      catch (error) {
          remove();
          callback(error, storeState, {}, remove);
      }
      return { remove: remove };
  }
  function connectCallback(dependencies, dispatcher) {
      if (dispatcher === void 0) { dispatcher = get(); }
      enforceDispatcher(dispatcher);
      if (!dispatcher) {
          return null;
      }
      return subscribe.bind(null, dependencies, makeDependencyIndex(dependencies), dispatcher);
  }

  function define() {
      return new StoreSingleton();
  }
  function defineFactory() {
      return new StoreFactory({});
  }

  exports.DispatcherInstance = DispatcherInstance;
  exports.InspectStore = InspectStore;
  exports.StoreFactory = StoreFactory;
  exports.connect = connect;
  exports.connectCallback = connectCallback;
  exports.define = define;
  exports.defineFactory = defineFactory;
  exports.useStoreDependency = useStoreDependency;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
