"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _DispatcherInterface = require("../dispatcher/DispatcherInterface");

var _Event = _interopRequireDefault(require("../event/Event"));

var _invariant = _interopRequireDefault(require("invariant"));

var _uniqueID = _interopRequireDefault(require("../uniqueid/uniqueID"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var HINT_LINK = 'Learn more about using the Store API:' + ' https://github.com/HubSpot/general-store#using-the-store-api';
var hasReduxDevTools = typeof window !== 'undefined' && window.__REDUX_DEVTOOLS_EXTENSION__;

function getNull() {
  return null;
}

var Store = /*#__PURE__*/function () {
  function Store(_ref) {
    var _this = this;

    var dispatcher = _ref.dispatcher,
        factory = _ref.factory,
        getter = _ref.getter,
        initialState = _ref.initialState,
        name = _ref.name,
        responses = _ref.responses;

    _classCallCheck(this, Store);

    _defineProperty(this, "_dispatcher", void 0);

    _defineProperty(this, "_dispatchToken", void 0);

    _defineProperty(this, "_factory", void 0);

    _defineProperty(this, "_getter", void 0);

    _defineProperty(this, "_event", void 0);

    _defineProperty(this, "_name", void 0);

    _defineProperty(this, "_responses", void 0);

    _defineProperty(this, "_state", void 0);

    _defineProperty(this, "_uid", void 0);

    _defineProperty(this, "_devToolsExtension", void 0);

    _defineProperty(this, "_unsubscribeDevTools", void 0);

    this._dispatcher = dispatcher;
    this._factory = factory;
    this._getter = getter;
    this._name = name || 'Store';
    this._state = initialState;
    this._responses = responses;
    this._event = new _Event.default();
    this._uid = (0, _uniqueID.default)();
    this._dispatchToken = this._dispatcher.register(this._handleDispatch.bind(this));

    if (hasReduxDevTools) {
      this._devToolsExtension = window.__REDUX_DEVTOOLS_EXTENSION__.connect({
        name: name || "Store_".concat(this._uid),
        instanceId: this._uid
      });
      this._unsubscribeDevTools = this._devToolsExtension.subscribe(function (message) {
        if (message.type === 'DISPATCH' && message.payload.type === 'JUMP_TO_ACTION') {
          _this._state = JSON.parse(message.state);

          _this.triggerChange();
        }
      });
    }
  }
  /**
   * Subscribe to changes on this store.
   *
   * @param  callback  will run every time the store responds to a dispatcher
   * @return this
   */


  _createClass(Store, [{
    key: "addOnChange",
    value: function addOnChange(callback) {
      (0, _invariant.default)(typeof callback === 'function', 'Store.addOnChange: expected callback to be a function' + ' but got "%s" instead. %s', callback, HINT_LINK);
      return this._event.addHandler(callback);
    }
    /**
     * Returns the store's referenced value
     *
     * @param  ...  accepts any number of params
     * @return any
     */

  }, {
    key: "get",
    value: function get() {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      return this._getter.apply(this, [this._state].concat(args));
    }
    /**
     * @protected
     * Responds to incoming messages from the Dispatcher
     */

  }, {
    key: "_handleDispatch",
    value: function _handleDispatch(payload) {
      if (process.env.NODE_ENV !== 'production') {
        (0, _invariant.default)((0, _DispatcherInterface.isPayload)(payload), 'Store: expected dispatched payload to be an object with a' + ' property "actionType" containing a string and an optional property' + ' "data" containing any value but got "%s" instead. Learn more about' + ' the dispatcher interface:' + ' https://github.com/HubSpot/general-store#dispatcher-interface');
      }

      var actionType = payload.actionType || payload.type;
      var data = payload.hasOwnProperty('data') ? payload.data : payload.payload;

      if (!actionType || !this._responses.hasOwnProperty(actionType)) {
        return;
      }

      this._state = this._responses[actionType](this._state, data, actionType, payload);
      this.triggerChange();

      if (this._devToolsExtension) {
        this._devToolsExtension.send(actionType, this._state);
      }
    }
    /**
     * Destroys this instance of the store.
     * Dispatch callback is unregistered. Subscriptions are removed.
     */

  }, {
    key: "remove",
    value: function remove() {
      this._dispatcher.unregister(this._dispatchToken);

      this._event.remove();

      this._getter = getNull;
      this._responses = {};
      typeof this._unsubscribeDevTools === 'function' && this._unsubscribeDevTools();
      typeof this._devToolsExtension !== 'undefined' && this._devToolsExtension.unsubscribe();
    }
  }, {
    key: "toString",
    value: function toString() {
      return "".concat(this._name, "<").concat(this._state, ">");
    }
    /**
     * Runs all of the store's subscription callbacks
     *
     * @return this
     */

  }, {
    key: "triggerChange",
    value: function triggerChange() {
      if (process.env.NODE_ENV !== 'production') {
        if (!this._dispatcher.isDispatching()) {
          console.warn('Store: you called store.triggerChange() outside of a' + ' dispatch loop. Send an action trough the dispatcher to' + ' avoid potentailly confusing behavior.');
        }
      }

      this._event.runHandlers();

      return this;
    }
  }]);

  return Store;
}();

exports.default = Store;